-- Set some long selection length, e.g. 44100 sample,
--  and execute 'make_wave(mod_sin_fn)'.
-- Enjoy FM synthesis !
function mod_sin_fn(x)
  local y =
  math.sin(2*math.pi*x*(400 + 0.5*math.sin(2*math.pi*800*x)))
  return y
end

----------------------------------------------

-- locals
local is_not_sliced
local init_sample_name =""
local init_inst , init_inst_index
local init_sample_index , init_sample , init_buffer

local init_interpolation_mode , init_new_note_action
local init_autoseek , init_autofade
local init_panning , init_volume
local init_loop_mode , init_loop_release

local init_loop_start , init_loop_end
local init_sample_rate , init_bit_depth
local init_number_of_channels , init_number_of_frames

local init_sync_enabled , init_sync_lines

-- base_note -> tranpose
-- http://forum.renoise.com/index.php?/topic/31452-base-note-always-returns-48/
local init_transpose , init_fine_tune

local new_sample , new_buffer
local upbeat_frames

local bpm, lpb, selection_start, selection_end



-- init sample settings

function get_init_env()

  init_inst = renoise.song().selected_instrument
  init_sample_index = renoise.song().selected_sample_index
  init_sample = renoise.song().selected_sample
  if init_sample ~= nil then
    init_buffer = renoise.song().selected_sample.sample_buffer
  end
  if init_sample == nil or
    init_buffer.has_sample_data ~= true
    then
      renoise.app():show_error(" No sample data.")
    return false
    end
  
  if
    #(renoise.song().selected_sample.slice_markers) >1
    then is_not_sliced = false
    else is_not_sliced = true
    end  

  init_sample_rate =
       init_buffer.sample_rate
  init_bit_depth =
       init_buffer.bit_depth
  init_number_of_channels =
  init_buffer.number_of_channels
  init_number_of_frames =
  init_buffer.number_of_frames

  init_interpolation_mode = init_sample.interpolation_mode
  init_new_note_action = init_sample.new_note_action
  init_autoseek = init_sample.autoseek
  init_autofade = init_sample.autofade
  init_panning = init_sample.panning
  init_volume = init_sample.volume
  init_loop_mode = init_sample.loop_mode
  init_loop_release = init_sample.loop_release
  
  init_sync_enabled = init_sample.beat_sync_enabled
  init_sync_lines = init_sample.beat_sync_lines

  init_sample_name = init_sample.name
  init_loop_start = init_sample.loop_start
  init_loop_end = init_sample.loop_end  
  init_transpose = init_sample.transpose
  init_fine_tune = init_sample.fine_tune

  selection_start =
          init_buffer.selection_start
  selection_end =
          init_buffer.selection_end
  bpm = renoise.song().transport.bpm
  lpb = renoise.song().transport.lpb
  return true
end

function not_sliced_ok ()
  if is_not_sliced == false
    then
    renoise.app():show_error(
               [[Sorry, sliced sample cannot be modified]])
    else return true
    end
end

-- new sample settings
function get_new_env()
  init_inst:insert_sample_at(init_sample_index +1)
  new_sample =
          init_inst.samples[init_sample_index +1]
          
  new_sample.beat_sync_enabled =  init_sync_enabled
  --  new_sample.beat_sync_lines =  init_sync_lines

  new_sample.transpose = init_transpose
  new_sample.fine_tune = init_fine_tune
  new_sample.name = init_sample_name

  new_sample.interpolation_mode = init_interpolation_mode
  new_sample.new_note_action = init_new_note_action
  new_sample.autoseek = init_autoseek
  new_sample.autofade = init_autofade
  new_sample.panning = init_panning
  new_sample.volume = init_volume
  new_sample.loop_mode = init_loop_mode
  new_sample.loop_release = init_loop_release

  new_buffer = new_sample.sample_buffer
  
end


function create_new (num_frames)
--  get_init_env()
--  init_inst:insert_sample_at(init_sample_index+1)
  get_new_env()

  local new_sync_val =
          init_sync_lines * (num_frames/init_number_of_frames) 
  if init_sync_enabled == false or new_sync_val > 256 
    then
      new_sample.beat_sync_lines = init_sync_lines
    else
      new_sample.beat_sync_lines = new_sync_val
  end 

  new_buffer:create_sample_data(
                 init_sample_rate,init_bit_depth,init_number_of_channels
                 , num_frames)
end


---------------------------------------------------------------

-- Draw the wave based on the fn (function)
function make_wave (fn)
  get_init_env()
  not_sliced_ok ()  
  local selection_range = selection_end - selection_start +1
  
  create_new (init_number_of_frames)

  new_buffer:prepare_sample_data_changes()

  for ch = 1,init_number_of_channels do
    for fr = 1,selection_start-1 do
      new_buffer
      :set_sample_data(
        ch,fr,
        init_buffer
        :sample_data(ch,fr))
    end
    for fr = selection_start,selection_end do
      new_buffer
      :set_sample_data(
        ch,fr,
        (fn((fr-selection_start)/selection_range)))
    end        
    for fr = selection_end+1,init_number_of_frames do
      new_buffer
      :set_sample_data(
        ch,fr,
        init_buffer
        :sample_data(ch,fr))  
    end
  end      
  new_buffer:finalize_sample_data_changes()
  
  new_sample.loop_start
  = init_loop_start
  new_sample.loop_end
  = init_loop_end        

  init_sample:copy_from(new_sample)
  init_inst:delete_sample_at(init_sample_index + 1)
  init_buffer.selection_start = selection_start
  init_buffer.selection_end = selection_end      
end


--  Multiply the selected sample data
--   by coefficients taking from the fn (function)
function set_fade (fn)
  get_init_env()
  not_sliced_ok ()  
  local selection_range = selection_end - selection_start +1
  
  create_new (init_number_of_frames)

  new_buffer:prepare_sample_data_changes()
  
  for ch = 1,init_number_of_channels do
    for fr = 1,selection_start-1 do
      new_buffer
      :set_sample_data(
        ch,fr,
        init_buffer
        :sample_data(ch,fr))
    end
    for fr = selection_start,selection_end do
      new_buffer
      :set_sample_data(
        ch,fr,
        (fn((fr-selection_start)/selection_range)*(init_buffer
          :sample_data(ch,fr))))
    end        
    for fr = selection_end+1,init_number_of_frames do
      new_buffer
      :set_sample_data(
        ch,fr,
        init_buffer
        :sample_data(ch,fr))  
    end
  end      
  new_buffer:finalize_sample_data_changes()
  
  new_sample.loop_start
  = init_loop_start
  new_sample.loop_end
  = init_loop_end        
  
  init_sample:copy_from(new_sample)
  init_inst:delete_sample_at(init_sample_index + 1)
  init_buffer.selection_start = selection_start
  init_buffer.selection_end = selection_end      
end

------------------------------------------


-- for test B
make_wave(mod_sin_fn)


